<?php

// Exit if accessed directly or class already exists
if ( ! defined( 'ABSPATH' ) || class_exists( 'Quiver_Delivery_Shipping_Method', false ) ) {
	return;
}

class Quiver_Delivery_Shipping_Method extends WC_Shipping_Method {

    private $METHOD_ID = 'quiver_delivery';
    private $CODE_DELIVERY_TYPE_MAP = [
		'QUI-IM' => 'IMMEDIATE', 
		'QUI-SD' => 'SAME_DAY', 
		'QUI-ND' => 'NEXT_DAY', 
		'QUI-DW' => 'DAWN', 
		'QUI-WN' => 'WINDOW'
	];
	private $BATCH_CREATE_QUOTE_URL = 'https://api.quiver.london/delivery/quotes:batchCreate';
	private $CREATE_DELIVERY_URL = 'https://api.quiver.london/delivery/deliveries';

	/**
	 * Logger
	 *
	 * @var QuiverDeliveryLogger
	 */
	protected $logger = null;


    /**
     * Init and hook in the integration.
     */
    public function __construct( $instance_id = 0 ) {
        $this->id = $this->METHOD_ID;
        $this->instance_id = absint( $instance_id );
        $this->method_title = __( 'Quiver Delivery', $this->METHOD_ID );
        $this->method_description = $this->METHOD_ID;

        $this->init();
    }
    /**
     * Initializes the instance.
     *
     * @since [*next-version*]
     */
    protected function init() {
        // Load the settings.
        $this->init_form_fields();
        $this->init_settings();

        add_action( 'woocommerce_update_options_shipping_' . $this->id, array( $this, 'process_admin_options' ) );
    }

    /**
     * Initialize integration settings form fields.
     *
     */
    public function init_form_fields() {

        $this->form_fields = array(
            'api_key' => array(
	            'title'             => __( 'API Key', $this->METHOD_ID ),
	            'type'              => 'text',
	            'description'       => __( 'This can be found by visiting app.quiver.co.uk and navigating to the Settings page.', $this->METHOD_ID ),
	            'desc_tip'          => true,
	            'default'           => '',
	            'placeholder'		=> '',
            ),
        );
    }

    /**
     * calculate_shipping function.
     *
     * @access public
     * @param mixed $package
     * @return void
     */
    public function calculate_shipping( $package = array() ) {
        if ($this->get_option('api_key', '') === '') {
            $this->log_msg('Please enter your Quiver API key to use the Quiver plugin. This can be found by visiting app.quiver.co.uk and navigating to the Settings page.');
        }

        if (is_checkout()) {

            $destination = $package['destination'];
            $address = [
                'line1' => array_key_exists('address_1', $destination) ? $destination['address_1'] : '',
                'line2' => array_key_exists('address_2', $destination) ? $destination['address_2'] : '',
                'city' => array_key_exists('city', $destination) ? $destination['city'] : '',
                'postcode' => array_key_exists('postcode', $destination) ? $destination['postcode'] : '',
            ];

            if (empty($address['postcode'])) {
                return;
            }
            
            $lineItems = [];
            foreach ( WC()->cart->get_cart() as $cart_item ) {
                $lineItem = ['productId' => $cart_item['data']->get_sku(), 'quantity' => $cart_item['quantity']];
                array_push($lineItems, $lineItem);
            }

            $order = [
                'subtotalPrice' => (float) WC()->cart->get_subtotal()
            ];

            $request = [
                'address' => $address,
                'lineItems' => $lineItems,
                'order' => $order
            ];

            $response = $this->post_request($this->BATCH_CREATE_QUOTE_URL, $request);
            if ($response) {
                $quotes = $response->quotes;
                if (count($quotes) > 0) {
                    usort($quotes, array($this, "quote_cmp"));
                    foreach( $quotes as $quote ) {
                        $this->add_rate( [
                            'id' => $quote->code,
                            'label' => $quote->detail->title,
                            'cost' => $quote->detail->price,
                            'calc_tax' => 'per_item',
                            'meta_data' => [
                                'code' => $quote->code,
                                'description' => $quote->detail->description,
                            ]
                        ]);
                    }
                } else {
                    $this->log_msg('Quiver unable to offer Quotes for order');
                    $this->log_msg(print_r($request, TRUE));
                }
            } else {
                $this->log_msg('Quiver unable to offer Quotes for order');
                $this->log_msg(print_r($request, TRUE));
            }
        }
    }

    

    public function create_delivery( $order_id ) {
		if ( ! $order_id )
			return;

		$order = wc_get_order($order_id);

		$shippingMethods = $order->get_shipping_methods();
		if (count($shippingMethods) === 1) {
			$shippingMethod = array_values($shippingMethods)[0];
			if ($shippingMethod->get_method_id() === $this->METHOD_ID) {
				$code = $shippingMethod->get_meta('code');
                
                $delivery_type = $this->CODE_DELIVERY_TYPE_MAP[$code];
                $quiver_order = $this->order_to_quiver_order($order);
                $quiver_line_items = $this->order_to_line_items($order);
                $quiver_customer = $this->order_to_quiver_customer($order);
                $quiver_purchaser = $this->order_to_quiver_purchaser($order);
                $quiver_address = $this->order_to_quiver_address($order);

                $request = [
                    'type' => $delivery_type,
                    'order' => $quiver_order,
                    'lineItems' => $quiver_line_items,
                    'customer' => $quiver_customer,
                    'address' => $quiver_address,
                ];

                if ($quiver_purchaser['firstName'] != $quiver_customer['firstName'] 
                    || $quiver_purchaser['lastName'] != $quiver_customer['lastName']
                    || $quiver_purchaser['company'] != $quiver_customer['company']) {
                    $request['purchaser'] = $quiver_purchaser;
                }

                $response = $this->post_request($this->CREATE_DELIVERY_URL, $request);
                if (!$response) {
                    $this->log_msg('Quiver unable to create delivery for order');
                    $this->log_msg(print_r($request, TRUE));
                }
			}
		}
	}
	
	/**
     * @param WC_Order $order
     * @return array
    */
	private function order_to_quiver_order( $order ) {
		return [
			'orderId' => strval($order->get_id()),
            'totalPrice' => (float) $order->get_total(),
            'subtotalPrice' => (float) $order->get_subtotal(),
            'note' => $order->get_customer_note(),
		];
	}

    	/**
     * @param WC_Order $order
     * @return array
    */
	private function order_to_line_items( $order ) {
		$lineItems = [];
		foreach ( $order->get_items()  as $item_id => $item ) {
			$product = $item->get_product();
			$lineItem = ['productId' => $product->get_sku(), 'quantity' => $item->get_quantity()];
			array_push($lineItems, $lineItem);
		}
        return $lineItems;
	}

	/**
     * @param WC_Order $order
     * @return array
    */
	private function order_to_quiver_address( $order ) {
		return [
			'line1' => $order->get_shipping_address_1(),
			'line2' => $order->get_shipping_address_2(),
			'city' => $order->get_shipping_city(),
			'postcode' => $order->get_shipping_postcode(),
		];
	}

	/**
     * @param WC_Order $order
     * @return array
    */
	private function order_to_quiver_customer( $order ) {
		return [
			'firstName' => $order->get_shipping_first_name(),
			'lastName' => $order->get_shipping_last_name(),
            'company' => $order->get_shipping_company(),
			'emailAddress' => $order->get_billing_email(),
			'phoneNumber' => $order->get_billing_phone()
		];
	}

    	/**
     * @param WC_Order $order
     * @return array
    */
	private function order_to_quiver_purchaser( $order ) {
		return [
			'firstName' => $order->get_billing_first_name(),
			'lastName' => $order->get_billing_last_name(),
            'company' => $order->get_billing_company(),
			'emailAddress' => $order->get_billing_email(),
			'phoneNumber' => $order->get_billing_phone()
		];
	}

    public function process_admin_options() {
        return parent::process_admin_options();
    }

    private function quote_cmp( $a, $b ) {
        return ($a->detail->price < $b->detail->price) ? -1 : 1;
    }


	private function log_msg( $msg )	{
		if( ! $this->logger ) {
			$this->logger = new QuiverDeliveryLogger();
		}
		$this->logger->write( $msg );
	}


    private function post_request( $url, $body ) { 
        $args = [
            'headers' => [
                'x-api-key' => $this->get_option('api_key', ''),
            ],
            'body' => wp_json_encode($body)
        ];
        $response = wp_remote_post($url, $args);


        if (! is_wp_error($response) ) {
            return json_decode(wp_remote_retrieve_body($response));
        } else {
            return false;
        }
    }
    
}